/*******************************************************************

PERIOD:

Ticker  Every 1.25 ms Timer0 interrupt increments the ticker.

    at (ticker= 1s + n*1s) period_error is set,
    at (ticker= 1.5s + n*1s) main reads the elapsed time of periods and their
       count, calculates the average period with (elapsed time/period count)         

The capture interrupt increments the period counter and
finishes the measuring with the end of the first period
after the period_error flag.

******************************************************************/

#include <reg167.h>
#include <stdio.h>
#include <math.h>
#include <intrins.h>

#pragma PD(0)

bit secflag;
bit period_error;
sbit P310 = P3^10; 
sbit DP310 = DP3^10;      

unsigned int idata ticker, period_count, overflow, overflow_mirror;
unsigned int idata period_count_local;

unsigned long idata sum_of_period_time;
typedef union { struct{unsigned int uint_l,uint_h;}i;
				unsigned long ulong;
      		  } int2long;	

int2long idata capture32; 
	
/******************************************************************/

void tickerint (void) interrupt 0x20
/* timer T0 overflow */
{
static unsigned int idata ticker;

  ++ticker;

   switch(ticker)
   {
      case 800:  period_error=1; break;

      case 1200: ticker=400; secflag=1; break;

   }
}
/******************************************************************/


void overflowint (void) interrupt 0x21
/* timer T1 overflow */
{
	++overflow;
}
/*****************************************************************/

void periodint (void) interrupt 0x11 using period_bank  // 0x11-CAPCOM
{                                                       // register 1

static unsigned long idata old_capture32;

++period_count_local;

if(period_error)
	{
	period_error=0;      
    
	capture32.i.uint_l=CC1;
	if (CC1<0x4000) capture32.i.uint_h=overflow_mirror+1;
	else if (CC1<0xC000) capture32.i.uint_h=overflow;
	else capture32.i.uint_h=overflow_mirror;

	sum_of_period_time = capture32.ulong-old_capture32;
	old_capture32 = capture32.ulong;
	period_count = period_count_local;
	period_count_local = 0;
	}
}

/******************************************************/


void main (void) {
// Initialisation

// Serial Port

   P310=1;         // set Port 3.10 output latch TxD 
   DP310=1;        // set Port 3.10 direction control TxD out      
   S0TIC=0x80;     // set transmit interrupt flag
   S0RIC=0x00;     // reset receive interrupt flag
   S0BG= 0x33;     // set baudrate 9600 @ 16 MC CPU clock
   S0CON=0x8011;   // set serial mode
                   


// Capcom Unit

CCM0=0x009E;     // CC Mode control reg; CC0 compare (interrupt only) enabled,
                 // CC0 allocated to T1,
		 // CC1 capture on rising edge at Pin CC1IO
		 // CC1 alocated to T1
CC0=0x8000;      // Compare at T1=0x8000

// Interrupts

T0IC=0x44;      // timer0 (ticker) int enable   ILVL=1 GLVL=0 
T1IC=0x45;      // timer1 (overflow count) int enable ILVL=1 GLVL=1 
CC1IC=0x48;     // capture CC1 int enable       ILVL=2 GLVL=0 


CC0IC=0x78;     // compare CC0 PEC0 int enable  ILVL=E PEC=0 


PECC0=0x00ff;   // PEC continuous 16 bit transfer  overflow_mirror <- overflow
SRCP0= _sof_( & overflow);
DSTP0= _sof_( & overflow_mirror);

period_count_local=overflow_mirror=0xFFFF;
T0REL=-2500;
T0=-2500;
T01CON=0x4140;   // x 1 xx 0 001   x 1 xx 0 000
		 // timer1 mode run enable, prescaler 16
         // timer0 mode run enable, prescaler 8
		 // autoreload -2500;
IEN=1;          // global interrupt enable 
period_error=1;
puts("OK\n");


while (1)
   {
      while(!secflag) _idle_();
      secflag=0;

      if(period_error)
	 	printf("error\n");
      else
	 
printf(" %7lu %6u period= %9.6f [ms]\n",sum_of_period_time,period_count,
        (float)sum_of_period_time/(float)period_count *0.001);
   }
}
